// ============================================================================
// CBP + FBO = MAS22 Simulation Framework
// 20-12-21
// 28-01-22
// MAS22_Actor.cpp
// 27-05-22
// 02-06-22
// 06-06-22
// 09-07-22 Global Log File
// 11-08-22 Default Solver set to RKF for all MAS22_Actors (via this config)
// 
// ============================================================================

#include "MAS22_Actor.h"
#include "MAS22_Structs.h"

DEFINE_LOG_CATEGORY(MAS22_Log);

// Sets default values
AMAS22_Actor::AMAS22_Actor() {

	PrimaryActorTick.bCanEverTick = false;

}

void AMAS22_Actor::ActorInitialize() {
	// config file stuff -----------------------------------------------------
	FString configFName = FPaths::GeneratedConfigDir().Append(TEXT("MAS22_System.ini"));
	//printf("AMAS22_Actor::ActorInitialize() path to config file = %s", *configFName);
	GConfig->LoadFile(configFName);
	GConfig->GetBool(TEXT("/Script/MAS22_SciencePark.MAS22_Actor"), TEXT("editorOverride"), editorOverride, configFName);

	GConfig->GetString(TEXT("/Script/MAS22_SciencePark.MAS22_Actor"), TEXT("defaultSolver"), defaultSolver, configFName);
	printf("MAS22_Actor::ActorInitialize() >>>>>Config File read: solver set to %s", *defaultSolver);
	if (defaultSolver == "RUNGE_KUTTA_FEHLBERG") solver = SolverEnum::RUNGE_KUTTA_FEHLBERG;

	//if (editorOverride == true) {
	//	printf(">>>>> editorOverride = true");
	//}


	// Set solver timer ------------------------------------------------------
	GetWorldTimerManager().SetTimer(TimerHandle, this, &AMAS22_Actor::ComputeDT, dT, true);
	//logTime = 0;

}


void AMAS22_Actor::InitGlobalLog() {
	// 09-07-22 Open global log file
	//FString logFile = FPaths::ProjectDir();
	FString fileName = "Saved/MAS22_Logs/";
	fileName.Append(actorName);
	fileName.Append(".log");
	flogFile.Append(fileName);
	logString.Append("Global Log for ");
	logString.Append(actorName);
	logString.Append("\n");
}

// USed for legacy models
void AMAS22_Actor::SetInitialConditions() {
	time = 0;
}

void AMAS22_Actor::SetModelParameters() {  }
//void AMAS22_Actor::BindModelParameters() {  }
void AMAS22_Actor::UpdateModelParameters() {  }
void AMAS22_Actor::GetDefaultParameters() { }

void AMAS22_Actor::SetModelICs() {
	time = 0.0;
}
//void AMAS22_Actor::BindModelICs() {  }
void AMAS22_Actor::UpdateModelICs() {  }

void AMAS22_Actor::GetActorModelParameters(TArray<modelParametersStruct>** pArray) {
	*pArray = &modelParameters;
}

void AMAS22_Actor::ReceiveParamUpdateFromPlayerController(TArray<modelParametersStruct> pArray) {
	int len = modelParameters.Num();
	for (int i = 0; i < len; i++) {
		modelParameters[i] = pArray[i];
	}
}

void AMAS22_Actor::GetActorModelICs(TArray<modelParametersStruct>** icArray) {
	*icArray = &modelInitialConditions;
}

void AMAS22_Actor::ReceiveICsUpdateFromPlayerController(TArray<modelParametersStruct> icArray) {
	int len = modelInitialConditions.Num();
	for (int i = 0; i < len; i++) {
		modelInitialConditions[i] = icArray[i];
	}
}

void AMAS22_Actor::SetParams(double paramArray[]) {  }

void AMAS22_Actor::Message() {  }						// Legacy devel/debug

void AMAS22_Actor::UserInput(int key) {  }

void AMAS22_Actor::F1_Pressed() {
	if (bRunning == true) {
		bRunning = false;
	}
	else {
		bRunning = true;
	}
}

void AMAS22_Actor::F2_Pressed() {  // Soft Reset
	SoftReset();
}

void AMAS22_Actor::F3_Pressed() {  // Hard Reset
	HardReset();
}

void AMAS22_Actor::SoftReset() {	// Maintains latest parameters
	bRunning = false;				// Resets ICs to defaults
	//SetInitialConditions();
	SetModelICs();
	InitializeSolver();
	Visualization(dT);
}

void AMAS22_Actor::HardReset() {	// Resets parameters to defaults
	bRunning = false;				// Resets ICs to defaults
	//SetInitialConditions();
	SetModelICs();
	//BindModelParameters();
	GetDefaultParameters();
	SetModelParameters();
	UpdateModelParameters();
	InitializeSolver();
	Visualization(dT);
}

void AMAS22_Actor::C_Pressed() {  }

void AMAS22_Actor::P_Pressed() {  }

void AMAS22_Actor::Key_Pressed(int32 key) {  }

void AMAS22_Actor::ToggleRunning() {
	bRunning = true;
}

void AMAS22_Actor::SetRunning(bool trueFalse) {
	bRunning = trueFalse;
}

void AMAS22_Actor::ComputeDT() {
	static bool dispDebug = true;
	if (dispDebug == true) {
		GEngine->AddOnScreenDebugMessage(-1, 4.0f, FColor::Purple,
			FString::Printf(TEXT(">>>> AMAS22_Actor::ComputeDT(): called: time = %f"), time));
		dispDebug = false;
	}
//	logTime += dT;
}

void AMAS22_Actor::Computation() {
	static bool dispDebug = true;
	if (dispDebug == true) {
		GEngine->AddOnScreenDebugMessage(-1, 4.0f, FColor::Purple,
			FString::Printf(TEXT(">>>> AMAS22_Actor::Computation(): called: time = %f"), time));
		dispDebug = false;
	}
}

void AMAS22_Actor::Euler(double deltaT) {}
void AMAS22_Actor::RKF(double deltaT) {}
void AMAS22_Actor::RK(double deltaT) {}

void AMAS22_Actor::BuildHUDLines() {
	HUDLine line;
	HUDLines.Empty();

	line.message = FString::Printf(TEXT("%s"), *actorName);
	line.colour = FColor::Red;
	line.value = 0;
	HUDLines.Add(line);

	FString solverName;
	if (solver == SolverEnum::EULER) solverName = "Euler";
	if (solver == SolverEnum::RUNGE_KUTTA) solverName = "Runge-Kutta";
	if (solver == SolverEnum::RUNGE_KUTTA_FEHLBERG) solverName = "Runge-Kutta-Fehlberg";

	line.message = FString::Printf(TEXT("Solver:  %s"), *solverName);
	line.colour = FColor::Yellow;
	line.value = 0;
	HUDLines.Add(line);

	line.message = FString::Printf(TEXT("time = %f"), time);
	line.colour = FColor::Red;
	line.value = 0;
	HUDLines.Add(line);

	for (int i = 0; i < modelParameters.Num(); i++) {
		line.message = FString::Printf(TEXT("%s = %f"), *modelParameters[i].name, modelParameters[i].val);
		line.colour = PARAM_COLOR;
		HUDLines.Add(line);
	}

}

void AMAS22_Actor::Visualization(float deltaTime) {  }

void AMAS22_Actor::Visualization(FVector vec) {  }

void AMAS22_Actor::GetHUDLines(TArray<HUDLine>** plines) {
	static bool dispDebug = true;

	*plines = &HUDLines;

	int len = (**plines).Num();
	if (dispDebug == true) {
		FString message = (**plines)[0].message;
		dispDebug = false;
	}

}

void AMAS22_Actor::LogData() {

}

void AMAS22_Actor::EndPlay(const EEndPlayReason::Type EndPlayReason) {
	Super::EndPlay(EndPlayReason);
	GetWorldTimerManager().ClearTimer(TimerHandle);
	// Write and close global log file
	FFileHelper::SaveStringToFile(logString, *flogFile);
}

void AMAS22_Actor::ApplyConfig() {

}

void AMAS22_Actor::InitializeSolver() {  }

//void AMAS22_Actor::Euler(double DeltaTime) {  }

//void AMAS22_Actor::EulerCromer(double DeltaTime) {  }

//void AMAS22_Actor::RungeKutta(double DeltaTime) {  }

//void AMAS22_Actor::RungeKuttaFehlberg(double DeltaTime) {  }

// Console params bound to modelParameters array -------------------------

void AMAS22_Actor::SetConsoleParam(struct FconsoleParam para) {
	// loop over parameters and search for the requested parameter
	FString requParaName = para.name;
	int foundParaIndex;
	for (int i = 0; i < modelParameters.Num(); i++) {
		if (modelParameters[i].name == requParaName) {
			foundParaIndex = i;
		}
	}
	if (modelParameters.Num() > 0) {
		modelParameters[foundParaIndex].val = para.val;
	}
}

// Additional console parameters for individual model --------------------

void AMAS22_Actor::SetConsoleAddit(FString name, FString strVal) { }

// System console commands, used by this

void AMAS22_Actor::SetConsoleSys(FString name, FString strVal) {
	printf("AMAS22_Actor::SetConsoleSys called (variant 2");
	if (name == "solver") {
		if (strVal == "RKF") {
			solver = SolverEnum::RUNGE_KUTTA_FEHLBERG;
		}
		if (strVal == "EULER") {
			solver = SolverEnum::EULER;
		}
		printf("AMAS22_Actor::SetConsoleSys dT set to %s", *strVal);
	}
	if (name == "dT") {
		dT = FCString::Atod(*strVal);
		printf("AMAS22_Actor::SetConsoleSys dT set to %f", dT);
	}
	if (name == "logdT") {
		logInterval = FCString::Atod(*strVal);
		printf("AMAS22_Actor::SetConsoleSys logInterval set to %f", logInterval)
	}

}


void AMAS22_Actor::DumpConsoleAll() {
	// dump modelParameters
	flog("\n------------------DumpConsoleAll() -------------------\n\n");
	flog("----------------- Defaults -------------------")
	for (int i = 0; i < modelParameters.Num(); i++) {
		FString name = modelParameters[i].name;
		double val = modelParameters[i].val;
		printf(">>>>> param %s = %e", *name, val);
		flog("param %s = %e\n", *name, val);
	}
	// dump modelInitialConditions
	for (int i = 0; i < modelInitialConditions.Num(); i++) {
		FString name = modelInitialConditions[i].name;
		double val = modelInitialConditions[i].val;
		printf(">>>>>IC %s = %e", *name, val);
		flog("IC %s = %e\n", *name, val);
	}
	// dump dTs'
	printf(">>>>> dT = %f", dT);
	printf(">>>>> logdT = %f", logInterval);
	printf(">>>>> scaling = %f", scaling);
	flog("dT = %f\n", dT);
	flog("logdT = %f\n", logInterval);
	flog("scaling = %f\n", scaling);

	// dump solver
	if (solver == SolverEnum::EULER) {
		printf(">>>>> solver = EULER");
		flog("solver = EULER\n");
	}
	if (solver == SolverEnum::RUNGE_KUTTA_FEHLBERG) {
		printf(">>>>> solver = RUNGE_KUTTA_FEHLBERG"); 
		flog("solver = RUNGE_KUTTA_FEHLBERG\n");
	}
	if (solver == SolverEnum::RUNGE_KUTTA) {
		printf(">>>>> solver = RUNGE_KUTTA");
		flog("solver = RUNGE_KUTTA\n");
	}
	// dump editorOverride
	if (editorOverride == true) {
		printf(">>>>> editorOverride = true");
		flog("editorOverride = true\n");
	}
	else {
		printf(">>>>> editorOverride = false");
		flog("editorOverride = false\n");
	}



}

