// ============================================================================
// CBP + FBO = MAS22 Simulation Framework
// 20-12-21
// MAS22_Actor.h
// 25-05-22, 27-05-22
// 02-06-22
// 06-06-22
// ============================================================================

#pragma once

#include "CoreMinimal.h"
#include "GameFramework/Actor.h"
#include "MAS22_Structs.h"
#include "MAS22_Actor.generated.h"

// 28-05-22 Experimental logging to the Ue4 system - creates new category MAS22_Log
DECLARE_LOG_CATEGORY_EXTERN(MAS22_Log, Log, All);
#define CUR_CLASS (FString(__FUNCTION__).Left(FString(__FUNCTION__).Find(TEXT(":"))) )
#define GETSTR(text, ...) *FString::Printf(TEXT(text), __VA_ARGS__)
#define LOG(text,...) UE_LOG(MAS22_Log,Warning, TEXT("%s:%s"),*CUR_CLASS,GETSTR(text, __VA_ARGS__))

// 09-07-22 Experimental flog
#define flog(x,...){logString.Append(FString::Printf(TEXT(x), __VA_ARGS__));}

// (12) -----------------------
UENUM()
	enum class SolverEnum : uint8 {
	EULER					UMETA(DisplayName = "Euler"),
//	EULER_CROMER			UMETA(DisplayName = "Euler-Cromer"),
	RUNGE_KUTTA				UMETA(DisplayName = "Runge-Kutta"),
	RUNGE_KUTTA_FEHLBERG	UMETA(DisplayName = "Runge-Kutta-Fehlberg"),
	SPECIAL					UMETA(DisplayName = "Special")
};

//UCLASS(Config = MAS22_System, defaultconfig, notplaceable)
UCLASS(Config = MAS22_System, defaultconfig)
class MAS22_SCIENCEPARK_API AMAS22_Actor : public AActor
{
	GENERATED_BODY()

public:
	AMAS22_Actor();
	// 27-05-22 Config file stuff
	UPROPERTY(Config)
		FString defaultSolver;

	//UPROPERTY(Config)
	//	double defaultDT;		// Probably don't need this

	// 09-07-22 Global log file ------------------------------------------
	FString flogFile = FPaths::ProjectDir();
	FString logString;


	UPROPERTY(Config)
		bool editorOverride;

	//09-07-22 Global Loggin
	virtual void InitGlobalLog();


	// 29-04-22 Console command stuff
	// 25-03-22 Now func is located in MAS22_Actor class
	virtual void SetConsoleParam(struct FconsoleParam para);
	virtual void SetConsoleAddit(FString name, FString strVal);
	virtual void SetConsoleSys(FString name, FString strVal);
	virtual void DumpConsoleAll();

	virtual void ActorInitialize();
	virtual void ToggleRunning();
	virtual void SetRunning(bool trueFalse);

	virtual void SetInitialConditions();

	virtual void SetModelParameters();
	//virtual void BindModelParameters();
	virtual void UpdateModelParameters();
	virtual void GetDefaultParameters();

	virtual void SetModelICs();
	//virtual void BindModelICs();
	virtual void UpdateModelICs();

	virtual void Euler(double dT);
	virtual void RKF(double dT);
	virtual void RK(double dT);

	virtual void SetParams(double paramArray[]);

	virtual void Message();

	// 26/02/2022
	virtual void UserInput(int key);
	virtual void F1_Pressed();
	virtual void F2_Pressed();
	virtual void F3_Pressed();
	virtual void C_Pressed();
	virtual void P_Pressed();
	virtual void Key_Pressed(int32 key);

	virtual void SoftReset();        // Should this be protected?
	virtual void HardReset();        // Should this be protected?

	//virtual void SetParam(FString name, double val);

	TArray<HUDLine> HUDLines;
	void GetHUDLines(TArray<HUDLine>** plines);

	int ID = 2;
	bool bRunning = false;
	bool bHUD_exists = false;

	FTimerHandle TimerHandle;

	// CBP 20-12-21 Name added. Used in HUD and to create unique log file
	UPROPERTY(EditAnywhere, Config, Category = "MAS22 Simulation Engine")
		FString actorName;

	// (12) -------------------------
	UPROPERTY(EditAnywhere, Category = "MAS22 Simulation Engine")
		SolverEnum solver;
	UPROPERTY(EditAnywhere, Config, Category = "MAS22 Simulation Engine")
		double dT = 0.01;

	// (18) --------------------------
	UPROPERTY(EditAnywhere, Category = "MAS22 Simulation Engine")
		double logInterval = 0.1;

	UPROPERTY(Config)
		double scaling = 100.0;

protected:
	virtual void LogData();
	virtual void EndPlay(const EEndPlayReason::Type EndPlayReason) override;
	virtual void ApplyConfig();

	virtual void ComputeDT();
	virtual void Computation();
	virtual void Visualization(float deltaTime);
	virtual void Visualization(FVector vec);

	double time;
	int itn;

public:

	virtual void InitializeSolver();
	//void Euler(double DeltaTime);
	//void EulerCromer(double DeltaTime);
	//void RungeKutta(double DeltaTime);
	//void RungeKuttaFehlberg(double DeltaTime);
	virtual void BuildHUDLines();

	//void SetParams(double paramArray[]);

	// for RKF --------------------------------------------------------------
	double* u;
	double* dudt;
	double* yout;
	double t;
	double tout;
	double relerr;
	double abserr;
	int flag;

	// LogTime allows logging of successive runs
	// double logTime;

	// Legacy Params (for separate rhs)
	double* parameters;

	// --- Parameters -------------------------------------------------------
	TArray<modelParametersStruct> modelParameters;
	void GetActorModelParameters(TArray<modelParametersStruct>** pArray);
	void ReceiveParamUpdateFromPlayerController(TArray<modelParametersStruct> pArray);

	// --- Initial Conditions -----------------------------------------------
	TArray<modelParametersStruct> modelInitialConditions;
	void GetActorModelICs(TArray<modelParametersStruct>** icArray);
	void ReceiveICsUpdateFromPlayerController(TArray<modelParametersStruct> icArray);



	// (21) options removed
	//double* options;

};
