// ============================================================================
// CBP + FBO = MAS22 Simulation Framework
// 20-12-21
// MAS22_HUD.cpp
// ============================================================================


#include "MAS22_HUD.h"
#include "Engine/Canvas.h"
#include "Engine/Texture2D.h"
#include "Engine.h"
#include "TextureResource.h"
#include "CanvasItem.h"
#include "UObject/ConstructorHelpers.h"

//14-5-22
//#include "SParamMenu.h"
#include "Widgets/SWeakWidget.h"
#include "GameFramework/PlayerController.h"

AMAS22_HUD::AMAS22_HUD() {
	static ConstructorHelpers::FObjectFinder<UTexture2D>
		CrosshairTexObj(TEXT("/Game/Textures/FirstPersonCrosshair"));
	Crosshair = CrosshairTexObj.Object;

	UObject* obj_ptr = StaticLoadObject(UFont::StaticClass(), NULL, TEXT("/Game/Fonts/Roboto"));
	UFont* TextFont = Cast<UFont>(obj_ptr);
	HUDFont = TextFont;
}


// CBP 14-05-2 -------------- can be removed - part of Widgets testing ----------------
void AMAS22_HUD::BeginPlay() {

	Super::BeginPlay();
	//ShowMenu();

}

// Following while learning Widgets, can be removed --------------------------------------
/*
void AMAS22_HUD::ShowMenu() {

	if (GEngine && GEngine->GameViewport) {
		ParamWidget = SNew(SParamMenu).OwningHUD(this);
		GEngine->GameViewport->AddViewportWidgetContent(SAssignNew(ParamWidgetContainer, SWeakWidget).PossiblyNullContent(ParamWidget.ToSharedRef()));
		if (PlayerOwner) {
			PlayerOwner->bShowMouseCursor = true;
			PlayerOwner->SetInputMode(FInputModeUIOnly());
		}
	}

}

void AMAS22_HUD::RemoveMenu() {
	printf("REMOVE MENU CALLED");
	if (GEngine && GEngine->GameViewport && ParamWidgetContainer.IsValid()) {
		GEngine->GameViewport->RemoveViewportWidgetContent(ParamWidgetContainer.ToSharedRef());
		if (PlayerOwner) {
			PlayerOwner->bShowMouseCursor = false;
			PlayerOwner->SetInputMode(FInputModeGameOnly());
		}
	}

}
*/
// ------------------------------------------------------

void AMAS22_HUD::DrawHUD() {
	Super::DrawHUD();

	// 14-05-22 Temp removal - playing with Slate
	//return;

	const FVector2D Center(Canvas->ClipX * 0.5f, Canvas->ClipY * 0.5f);

	const FVector2D CrosshairDrawPosition((Center.X - 8), (Center.Y - 8));

	FCanvasTileItem TileItem(CrosshairDrawPosition, Crosshair->Resource, FLinearColor::White);
	TileItem.BlendMode = SE_BLEND_Translucent;
	Canvas->DrawItem(TileItem);

	dispText();

	if (bActorIsSelected == true) {
		DispSelectedActorInfo();
	}

}

void AMAS22_HUD::dispText() {
	FString HUDString = FString::FromInt(valToDisp);

	UObject* obj_ptr = StaticLoadObject(UFont::StaticClass(), NULL, TEXT("/Game/Fonts/Roboto"));
	UFont* TextFont = Cast<UFont>(obj_ptr);

	float XLength = 1.0f;
	float YLength = 3.0f;

	Canvas->TextSize(TextFont, HUDString, XLength, YLength);

	float X = Canvas->ClipX * 0.9f + XLength;
	float Y = Canvas->ClipY * 0.2f + YLength;

	FColor TextColor = FColor::Red;

	FCanvasTextItem TextItem(FVector2D(X, Y), FText::FromString(HUDString), TextFont, FColor(TextColor));
	//TextItem.bOutlined = true;
	//TextItem.OutlineColor = FLinearColor::Yellow;

	// Below successfully scales the large red number.
	FVector2D FontScale(5.0f, 5.0f);
	TextItem.Scale = FontScale;

	Canvas->DrawItem(TextItem);

}

void AMAS22_HUD::DispSelectedActorInfo() {
	TArray<HUDLine>* lines;

	if (!selectedActor) {
		GEngine->AddOnScreenDebugMessage(-1, 4.0f, FColor::Red,
			FString::Printf(TEXT("ERROR: Null Pointer")));

		return;
	}

	if (selectedActor->bHUD_exists == false) {
		return;
	}

	selectedActor->GetHUDLines(&lines);

	int len = (*lines).Num();

	for (int i = 0; i < len; i++)
	{
		DrawText((*lines)[i].message, (*lines)[i].colour, 800, 100 + 20 * i, HUDFont);
	}
}

void AMAS22_HUD::SetSelectedActor(AMAS22_Actor* Masac) {
	selectedActor = Masac;

	if (selectedActor) {
		bActorIsSelected = true;
	}

	GEngine->AddOnScreenDebugMessage(-1, 5.0f, FColor::Red,
		FString::Printf(TEXT("AMAS22_HUD::SetSelectedActor ID = %d"), Masac->ID));

	if (selectedActor->bHUD_exists == false) {
		return;
	}

	TArray<HUDLine>* lines;
	Masac->GetHUDLines(&lines);

	// CreateParamMenu();

}

void AMAS22_HUD::SetTo(int pValue) {
	valToDisp = pValue;
}

