// ============================================================================
// CBP + FBO = MAS22 Simulation Framework
// 20-12-21
// MAS22_PlayerController.cpp
// 03-02-22 CBP
// 05-03-22 FBO CBP
// 30-04-22 CBP exec function setParam added to allow use of console commands
// 19-05-22 Slate Interface added
// 27-05-22 Slate susbsytem tweaked
// 31-05-22 selectedActor->UpdateModelParameters(); added
// 02-06-22 Console now has setParam, setAddit and setSys
// ============================================================================


#include "MAS22_PlayerController.h"
#include "SlateBasics.h"
#include "SlateOptMacros.h"
#include "MAS22_Pawn.h"
#include "Kismet/GameplayStatics.h"
#include "Kismet/KismetSystemLibrary.h"
#include "DrawDebugHelpers.h"
#include "EngineUtils.h"
#include "Engine/World.h"
#include "MAS22_Actor.h"
#include "MAS22_HUD.h"
#include "MAS22_Structs.h"
#include "SParamMenuWidget.h"
#include "SICsMenuWidget.h"
#include "Widgets/SWeakWidget.h"


AMAS22_PlayerController::AMAS22_PlayerController() : APlayerController() {
	PrimaryActorTick.bCanEverTick = true;
	showIt = false;
}

void AMAS22_PlayerController::BeginPlay() {

	Super::BeginPlay();

	Pawn = Cast<AMAS22_Pawn>(GetPawn());
	if (!Pawn) {
		GEngine->AddOnScreenDebugMessage(-1, 5.0f, FColor::Blue,
			FString::Printf(TEXT("NO PAWN")));
	}

	// Started a timer to call Update()
	GetWorldTimerManager().SetTimer(TimerHandle, this,
		&AMAS22_PlayerController::Update, 1.0, true);

	// Places time as a UI element (testing) -------------------------------------
	// Lambda allows for dynamic updating
	TSharedRef<SVerticalBox> widget1 = SNew(SVerticalBox)
		+ SVerticalBox::Slot().HAlign(HAlign_Left).VAlign(VAlign_Center)[
			SNew(SButton).Content()[
				SNew(STextBlock).Text_Lambda([this] {return(FText::AsNumber(itn)); })
			]
		];
	GEngine->GameViewport->AddViewportWidgetForPlayer(GetLocalPlayer(), widget1, 1);

	HUD = Cast<AMAS22_HUD>(this->GetHUD());

	if (!HUD) {
		GEngine->AddOnScreenDebugMessage(-1, 5.0f, FColor::Blue,
			FString::Printf(TEXT("NO HUD")));

		return;
	}

}

void AMAS22_PlayerController::Update() {
	itn++;
	HUD->SetTo(itn);
}

void AMAS22_PlayerController::SetupInputComponent() {
	Super::SetupInputComponent();

	InputComponent->BindAction("Fire", IE_Pressed, this, &AMAS22_PlayerController::ActivateObject);
	InputComponent->BindAction("N_Key", IE_Pressed, this, &AMAS22_PlayerController::N_Pressed);
	InputComponent->BindAction("M_Key", IE_Pressed, this, &AMAS22_PlayerController::M_Pressed);
	InputComponent->BindAction("P_Key", IE_Pressed, this, &AMAS22_PlayerController::P_Pressed);
	InputComponent->BindAction("I_Key", IE_Pressed, this, &AMAS22_PlayerController::I_Pressed);

	// CBP 07-05-22 Following the approach below, we only need a single callback function which will
	// handle any keyboard key
	InputComponent->BindAction("Up_Key", IE_Pressed, this, &AMAS22_PlayerController::AKey_Pressed<KeyCodes::Up>);
	InputComponent->BindAction("Down_Key", IE_Pressed, this, &AMAS22_PlayerController::AKey_Pressed<KeyCodes::Down>);
	InputComponent->BindAction("Left_Key", IE_Pressed, this, &AMAS22_PlayerController::AKey_Pressed<KeyCodes::Left>);
	InputComponent->BindAction("Right_Key", IE_Pressed, this, &AMAS22_PlayerController::AKey_Pressed<KeyCodes::Right>);
	InputComponent->BindAction("C_Key",  IE_Pressed, this, &AMAS22_PlayerController::AKey_Pressed<KeyCodes::C>);

	InputComponent->BindAction("F1_Key", IE_Pressed, this, &AMAS22_PlayerController::F1_Pressed);
	InputComponent->BindAction("F2_Key", IE_Pressed, this, &AMAS22_PlayerController::F2_Pressed);
	InputComponent->BindAction("F3_Key", IE_Pressed, this, &AMAS22_PlayerController::F3_Pressed);

}

void AMAS22_PlayerController::N_Pressed() {
	AMAS22_Actor* Masac;
	for (TActorIterator<AMAS22_Actor> ActorItr(GetWorld()); ActorItr; ++ActorItr) {
		Masac = *ActorItr;
		Masac->UserInput(-1);
	}
}

void AMAS22_PlayerController::M_Pressed() {
	AMAS22_Actor* Masac;
	for (TActorIterator<AMAS22_Actor> ActorItr(GetWorld()); ActorItr; ++ActorItr) {
		Masac = *ActorItr;
		Masac->UserInput(1.0);
	}
}

void AMAS22_PlayerController::F1_Pressed() {
	if (selectedActor != NULL) {
		selectedActor->F1_Pressed();
	}	
}

void AMAS22_PlayerController::F2_Pressed() {		// Soft Reset
	if (selectedActor != NULL) {
		selectedActor->F2_Pressed();
	}
}

void AMAS22_PlayerController::F3_Pressed() {		// Hard Reset
	if (selectedActor != NULL) {
		selectedActor->F3_Pressed();
	}
}

// Below are legacy ------------------------------------------------------
/*
void AMAS22_PlayerController::C_Pressed() {
	if (selectedActor != NULL) {
		selectedActor->C_Pressed();
	}
}
*/
/*
void AMAS22_PlayerController::C_Pressed() {
	if (selectedActor != NULL) {
		selectedActor->Key_Pressed(KeyCodes::C);
	}
}
*/

void AMAS22_PlayerController::AKey_Pressed(int32 index) {
	//int32 iindex = index;
	printf("PC: Key pressed = %d\n", index);
	if (selectedActor != NULL) {
		selectedActor->Key_Pressed(index);
	}
}

// New Parameter menu  ---------------------------------------------------
void AMAS22_PlayerController::P_Pressed() {
	if (selectedActor) {
		ShowMenuParameters();
	}
}

// New P menu  InitialConditions --------------------------------------------------
void AMAS22_PlayerController::I_Pressed() {
	if (selectedActor) {
		ShowMenuInitialConditions();
	}
}

void AMAS22_PlayerController::ActivateObject() { // ----------------------

	// First turn off any running actor
	AMAS22_Actor* Masacs;
	
	for (TActorIterator<AMAS22_Actor> ActorItr(GetWorld()); ActorItr; ++ActorItr) {
		Masacs = *ActorItr;
		Masacs->SetRunning(false);
	}

	FPCC = Pawn->FirstPersonCameraComponent;

	FVector start = FPCC->GetComponentLocation();

	FVector end = start + FPCC->GetForwardVector() * 100000000;

	//DrawDebugLine(GetWorld(), start, end, FColor::Purple, true, 1.0f, (uint8)'\000', 5.0f);

	TEnumAsByte<ETraceTypeQuery> Actor = ETraceTypeQuery::TraceTypeQuery1;
	TArray<AActor*> Ignored;
	Ignored.Add(this);
	FHitResult Hit(ForceInit);
	bool bHitActor = UKismetSystemLibrary::LineTraceSingle(
		GetWorld(), start, end, Actor, false, Ignored, EDrawDebugTrace::ForDuration, Hit, true);

	if (bHitActor) {
		if (Hit.Actor->IsA(AMAS22_Actor::StaticClass())) {
			AMAS22_Actor* Masac = Cast<AMAS22_Actor>(Hit.GetActor());
			HUD->SetSelectedActor(Masac);
			selectedActor = Masac;
		}
	}

}

// ------------- CBP 16-05-22 Slate Menu Stuff ---------------------------

// Ccalled if selectedActor is not null!! Done in response to P-key press

void AMAS22_PlayerController::ShowMenuParameters() {
	selectedActor->GetActorModelParameters(&modelParamArray);
	ParamMenuWidget = SNew(SParamMenuWidget).OwningPlayerController(this).title("Parameters");
	GEngine->GameViewport->AddViewportWidgetContent(SAssignNew(ParamMenuWidgetContainer, SWeakWidget).PossiblyNullContent(ParamMenuWidget.ToSharedRef()));
	this->bShowMouseCursor = true;
	this->SetInputMode(FInputModeUIOnly());
}

void AMAS22_PlayerController::ShowMenuInitialConditions() {
	selectedActor->GetActorModelICs(&modelICsArray);
	ICsMenuWidget = SNew(SICsMenuWidget).OwningPlayerController(this).title("Initial Conditions");
	GEngine->GameViewport->AddViewportWidgetContent(SAssignNew(ICsMenuWidgetContainer, SWeakWidget).PossiblyNullContent(ICsMenuWidget.ToSharedRef()));
	this->bShowMouseCursor = true;
	this->SetInputMode(FInputModeUIOnly());
}

void AMAS22_PlayerController::RemoveMenu() {
	printf("REMOVE MENU CALLED");
	if (GEngine && GEngine->GameViewport && ICsMenuWidgetContainer.IsValid()) {
		GEngine->GameViewport->RemoveViewportWidgetContent(ICsMenuWidgetContainer.ToSharedRef());
		this->bShowMouseCursor = false;
		this->SetInputMode(FInputModeGameOnly());
	}
	if (GEngine && GEngine->GameViewport && ParamMenuWidgetContainer.IsValid()) {
		GEngine->GameViewport->RemoveViewportWidgetContent(ParamMenuWidgetContainer.ToSharedRef());
		this->bShowMouseCursor = false;
		this->SetInputMode(FInputModeGameOnly());
	}
}


//void AMAS22_PlayerController::ReceiveParamFromLineWidget(int id, float paramVal) {
//	printf("AMAS22_PlayerController::ReceiveParamFromLineWidget() id = %d, val = &f", id, paramVal);
//}

// First set up the parameter menu ---------------------------------------

void AMAS22_PlayerController::ReceiveParamsFromMenuWidget(TArray<modelParametersStruct>* pArray) {
	int EntriesNum = (*pArray).Num();
	// Now must update the selectedActor parameters by direct copy.
	int len = modelParamArray->Num();
	for (int ii = 0; ii < len; ii++) {
		modelParamArray[ii] = pArray[ii];
	}
	if (selectedActor) {
		selectedActor->BuildHUDLines();
		selectedActor->UpdateModelParameters();
	}
}

void AMAS22_PlayerController::GetParamArray(TArray<modelParametersStruct>** pArray) {
	*pArray = modelParamArray;
}

// Now the initial conditions menu ---------------------------------------

void AMAS22_PlayerController::ReceiveICsFromMenuWidget(TArray<modelParametersStruct>* icArray) {
	int EntriesNum = (*icArray).Num();
	// Now must update the selectedActor parameters by direct copy.
	int len = modelICsArray->Num();
	for (int ii = 0; ii < len; ii++) {
		modelICsArray[ii] = icArray[ii];
	}
	if (selectedActor) {
		selectedActor->BuildHUDLines();
		selectedActor->UpdateModelICs();
	}
}

void AMAS22_PlayerController::GetICsArray(TArray<modelParametersStruct>** icArray) {
	*icArray = modelICsArray;
}

// ------------- End Slate Menu Stuff ------------------------------------


// Console input ---------------------------------------------------------
// 02-06-22 Refactored so only SetParam takes string and double as param
// Others take 2 strings, to be converted within MAS22_Actor

void AMAS22_PlayerController::SetParam(FString _name, double _val){
	FconsoleParam ccParam;
	ccParam.name = _name;
	ccParam.val = _val;
	if(selectedActor) selectedActor->SetConsoleParam(ccParam);
	if(selectedActor) selectedActor->UpdateModelParameters();
}

void AMAS22_PlayerController::SetAddit(FString _name, FString _strVal) {
	if (selectedActor) selectedActor->SetConsoleAddit(_name, _strVal);
}

 void AMAS22_PlayerController::SetSys(FString _name, FString _strVal) {
	if (selectedActor) selectedActor->SetConsoleSys(_name, _strVal);
}

 void AMAS22_PlayerController::DumpAll () {
	 if (selectedActor) selectedActor->DumpConsoleAll();
 }

