// ============================================================================
// CBP + FBO = MAS22 Simulation Framework
// 20-12-21
// MAS22_PlayerController.h
// 05-03-22 FBO CBP
// 30-04-22 CBP Exec function added.
// 02-06-22
// ============================================================================

#pragma once

#include "CoreMinimal.h"
#include "GameFramework/PlayerController.h"
#include "Camera/CameraComponent.h"
#include "MAS22_HUD.h"
#include "MAS22_Structs.h"
#include "MAS22_PlayerController.generated.h"


UCLASS()
class MAS22_SCIENCEPARK_API AMAS22_PlayerController : public APlayerController
{
	GENERATED_BODY()
	
		// For console input bound with modelParameters array via MAS22_Actor
		UFUNCTION(Exec, Category = ExecFunctions)
		void SetParam(FString name, double val);

		// Console input for addit params outside modelParameters array
		UFUNCTION(Exec, Category = ExecFunctions)
		void SetAddit(FString name, FString strVal);

		// Console input for system params such as dT dLogT
		UFUNCTION(Exec, Category = ExecFunctions)
		void SetSys(FString name, FString strVal);

		// Dumps all model parameters, addit and sys params
		UFUNCTION(Exec, Category = ExecFunctions)
			void DumpAll();


public:
	// Sets default values for this actor's properties
	AMAS22_PlayerController();

	FTimerHandle TimerHandle;
	void Update();
	int itn;
	int i;

	virtual void SetupInputComponent() override;

	class AMAS22_Pawn* Pawn;

protected:
	// Called when the game starts or when spawned
	virtual void BeginPlay() override;

private:
	void ActivateObject();
	void N_Pressed();
	void M_Pressed(); 
	void F1_Pressed();
	void F2_Pressed();
	void F3_Pressed();
	void P_Pressed();
	void I_Pressed();
	void C_Pressed();

	// Special ------------------------------------------------------
	void AKey_Pressed(int32 index);
	template<int32 Index>
	void AKey_Pressed(){
		AKey_Pressed(Index);
	}

	AMAS22_HUD* HUD;

	UCameraComponent* FPCC;

	AMAS22_Actor* selectedActor;

	TSharedRef<SWidget> CreateParamMenu();

	bool showIt;

	bool widgetCreated = false;

	struct FconsoleParam cParam;

	void ButtClicked();

protected:
	// CBP 15-05-22
	TSharedPtr<class SParamMenuWidget> ParamMenuWidget;
	TSharedPtr<class SWidget>    ParamMenuWidgetContainer;

	// CBP 08-06-22
	TSharedPtr<class SICsMenuWidget> ICsMenuWidget;
	TSharedPtr<class SWidget>    ICsMenuWidgetContainer;

public:
	// CBP 15-05-22. 08-06-22
	void ShowMenuParameters();
	void ShowMenuInitialConditions();
	void RemoveMenu();

	//17-05-22
	void ReceiveParamFromLineWidget(int id, float paramVal);

	void ReceiveParamsFromMenuWidget(TArray<modelParametersStruct>* pArray);
	void ReceiveICsFromMenuWidget(TArray<modelParametersStruct>* icArray);

	int GetParamArrayLength();

	void GetParamArray(TArray<modelParametersStruct>** pArray);
	void GetICsArray(TArray<modelParametersStruct>** icArray);

	double* dummyParams;

	TArray<modelParametersStruct>* modelParamArray;
	TArray<modelParametersStruct>* modelICsArray;
};
