// ============================================================================
// CBP + FBO = MAS22 Simulation Framework
// 20-12-21
// MAS22_Structs.h
// CBP 30-04-22 struct consoleParam added
// CBP 24-05-22 Additional print and log macros from RIE_V2
// 02-06-22
// ============================================================================

#pragma once

#include "CoreMinimal.h"
#include "EngineGlobals.h"
#include "Engine/Engine.h"
//#include "MAS22_GameMode.h"
#include "MAS22_Structs.generated.h"

// macros for simplied on-screen debugging
#define printf(x, ...) if(GEngine){GEngine->AddOnScreenDebugMessage(-1, 15.0f, FColor::Blue, FString::Printf(TEXT(x), __VA_ARGS__));}
#define print(Text) if(GEngine) GEngine->AddOnScreenDebugMessage(-1,2.0f,FColor::White,TEXT(Text), false)
#define printErr(x, ...) if(GEngine){GEngine->AddOnScreenDebugMessage(-1, 15.0f, FColor::Red, FString::Printf(TEXT(x), __VA_ARGS__));}
#define printYell(x, ...) if(GEngine){GEngine->AddOnScreenDebugMessage(-1, 15.0f, FColor::Yellow, FString::Printf(TEXT(x), __VA_ARGS__));}
#define printOrange(x, ...) if(GEngine){GEngine->AddOnScreenDebugMessage(-1, 15.0f, FColor::Orange, FString::Printf(TEXT(x), __VA_ARGS__));}


// macros for writing to log file using string.Append(logf(....));
#define logf(x, ...) FString::Printf(TEXT(x), __VA_ARGS__)



// 28-05-22 Experimental logging to the Ue4 system --------- moved into MAS22_Actor -----------
//#define CUR_CLASS (FString(__FUNCTION__).Left(FString(__FUNCTION__).Find(TEXT(":"))) )
//#define GETSTR(text, ...) *FString::Printf(TEXT(text), __VA_ARGS__)
//#define LOG(text,...) UE_LOG(LogTemp,Log, TEXT("%s:%s"),*CUR_CLASS,GETSTR(text, __VA_ARGS__))

#define M_PI (4.0*atan(1.0))

// Colors for HUD
#define PARAM_COLOR FColor::Blue
#define VARIBLE_COLOR FColor::Green

// 
// For key input via player controller -----------------------------------
enum KeyCodes {
	A,
	B,
	C,
	Up,
	Down,
	Left,
	Right
};

// For HUD ---------------------------------------------------------------
struct HUDLine {
	FString message;
	FColor colour;
	double value;
};

// For N-body simulation -------------------------------------------------
struct scaledBodyInfo {
		FVector pos;
		FVector vely;
		double mass;
		FString name;
		FVector accel;
		double size;
};

// For console commands --------------------------------------------------

USTRUCT() struct FconsoleParam {
	GENERATED_USTRUCT_BODY()
	UPROPERTY() 
		FString name;
	UPROPERTY() 
		double val;
};

// Support of parameter - Slate subsystem --------------------------------
struct modelParametersStruct {
	FString name;
	double val;
};
	
// Example of how to use log macro ---------------------------------------
/*
void ARIE22_SceneManager::DumpWaypointArray() {
	FString logFile = FPaths::ProjectDir();
	FString fileName = "CBP_FBO_DebugLogs/";
	fileName.Append("Waypoints");
	fileName.Append(".txt");
	logFile.Append(fileName);

	FString str;

	for (int i = 0; i < TGInfoArray.Num(); i++) {
		str.Append(logf("Target %d Id = %d pointer = %p \n",
		i,TGInfoArray[i].Id,&(TGInfoArray[i].actor)));
	}
	FFileHelper::SaveStringToFile(str, *logFile);
}
*/