// =======================================================================
// SParamMenuWidget.cpp
// CBP 15-05-22
// class for Slate experiments
// Rev 19-05-22
// =======================================================================
#include "SParamMenuWidget.h"
#include "SParamLineWidget.h"
#include "MAS22_PlayerController.h"
#include "Widgets/SWeakWidget.h"
#include "Framework/Commands/UIAction.h"
#include "MAS22_Structs.h"

#define LOCTEXT_NAMESPACE "MainMenuWidget"

int id;

void SParamMenuWidget::Construct(const FArguments& InArgs) {
	const bool CloseAfterSelection = true;
	bCanSupportFocus = true;

	OwningPlayerController = InArgs._OwningPlayerController;
	FString title = InArgs._title;

	OwningPlayerController->GetParamArray(&pArray);
	EntriesNum = (*pArray).Num();
	int len = EntriesNum;
	//printf("SParamMenuWidget::Construct(): EntriesNum = %d", EntriesNum);
	//struct modelParametersStruct line;

	//for (int i = 0; i < len; i++) {
	//	printf("SParamMenuWidget::Construct(): Parameter %d = %f", i, (*pArray)[i].val);
	//}
	

	FMenuBuilder MenuBuilder(CloseAfterSelection, NULL);

	// Now build the menu ----------------------------------------------
	MenuBuilder.BeginSection("TestSection");

	MenuBuilder.AddWidget(
		SNew(STextBlock)
		//.Text(FText::FromString("Parameter Menu")),
		.Text(FText::FromString(title)),
		FText(),
		true);


	for (int i = 0; i < EntriesNum; i++) {
	
		TSharedRef<SParamLineWidget>PLineWidget = SNew(SParamLineWidget);

		PLineWidget->SetID(i);
		PLineWidget->SetVal((*pArray)[i].val);
		PLineWidget->SetMenuWidget(this);

		FString lineLabel = (*pArray)[i].name;

		MenuBuilder.AddWidget(PLineWidget, FText::FromString(lineLabel));

	}

	TSharedRef<SWidget> button =
		SNew(SHorizontalBox)
		+ SHorizontalBox::Slot()
		.AutoWidth()
		[
			SNew(SButton)
			.Text(FText::FromString("Done"))
			.OnClicked(this, &SParamMenuWidget::OnPlayClicked)
		];
		MenuBuilder.AddWidget(button, FText::GetEmpty());

	MenuBuilder.EndSection();

	TSharedRef<SWidget> MenuBarWidget = MenuBuilder.MakeWidget();

	ChildSlot
		[
			SNew(SVerticalBox)
			+ SVerticalBox::Slot()
			.HAlign(HAlign_Left)
			.VAlign(VAlign_Center)
			[
				MenuBarWidget
			]
		];

}

// Get all params from the lines and pack off to PCont here
FReply SParamMenuWidget::OnPlayClicked() {

	printf("SParamMenuWidget::OnPlayClicked()");

	if (OwningPlayerController.IsValid()) {
		PushParamsToPlayerController();
		OwningPlayerController->RemoveMenu();
	}
	return FReply::Handled();

}

void SParamMenuWidget::SpinBoxChanged(float newVal) {
	printf("SParamMenuWidget::SpinBoxChange() to %f",newVal);
}

void SParamMenuWidget::SpinBoxID(int iid) {
	printf("Spin Box %d", iid);
}

void SParamMenuWidget::PushParamsToPlayerController() {

	OwningPlayerController->ReceiveParamsFromMenuWidget(pArray);
}

void SParamMenuWidget::UpdateLine(int iden, float val) {
	(*pArray)[iden].val = val;
}

#undef LOCTEXT_NAMESPACE
